---------------------------------------------------------------------------------------------------
--
-- Name:		Data path module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	data_path.vhd
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		i_reset
--					Reset for data path (places data buses in high impedence)
--				i_ep_wr
--					Asserted during External Port write access
--				i_ep_rd
--					Asserted during External Port read access
--				i_hp_wr
--					Asserted during Host Port write access
--				i_latch_had
--					Asserted when data on HAD can be latched;
--					Data are latched internally when this signal is high, and output to DATA[15..0]
--					when it is low.
--				i_address[3..0]
--					External Port address bus
--
-- In/Outputs:	io_data[15..0]
--					External Port data bus
--				io_had[15..0]
--					Host Port multiplexed address/data bus
--
-- Outputs:		o_ha16
--					HP signal HA16
--				o_valid_hp_access
--					Signal is asserted during valid access to Host Port
--					(see table below for valid addresses and valid access types [R/W])
--
-- Description:	This module implements the data path between Host DSP and Target DSP. The input
--				signals i_ep_wr, i_ep_rd, i_hp_wr and i_latch_had are used to determine the
--				direction of the data flow.
--				The shifting of values on bus HAD is done following the address generation scheme
--				described in Chapter 8, Host Port, of the ADSP-2191 Hardware Reference.
--				The following table shows the address space used inside the PLD. Accesses to
--				addresses 0 through 5 trigger a Host Port access. A read from address 8 (register
--				VERSION) returns the version of the PLD.
--
--				   PLD Address  |    Access type    |  Read/Write
--				 -------------------------------------------------
--				        0       |   HP DM ADDRESS   |      W
--				        1       |   HP DM DATA      |     R/W
--				        2       |   HP PM ADDRESS   |      W
--				        3       |   HP PM DATA      |     R/W
--				        4       |   HP IO ADDRESS   |      W
--				        5       |   HP IO DATA      |     R/W
--				        6       |   Reserved        |      -
--				        7       |   Reserved        |      -
--				        8       |   REG_VERSION     |      R
--				      9 - 15    |   Reserved        |      -
--
-- NOTE:		The addresses shown above are not for the Target DSP, but are used inside the PLD
--				to detect different access types to the Host Port (Address/Data, MEM/IO, PM/DM)!
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE work.pack_const.ALL;

-- entity for data path
ENTITY data_path IS
	PORT(
		-- control signals for data path
		i_reset				: IN	STD_LOGIC;	-- reset for latches in data path
		i_latch_had			: IN	STD_LOGIC;	-- latch data on HAD[15..0] internally
		o_valid_hp_access	: OUT	STD_LOGIC;	-- asserted for access to valid address space

		-- External Port signals
		io_data				: INOUT	STD_LOGIC_VECTOR(15 DOWNTO 0);	-- DATA[15..0]
		i_address			: IN	STD_LOGIC_VECTOR(3 DOWNTO 0);	-- ADDRESS[3..0]
		i_ep_wr				: IN	STD_LOGIC;	-- WR (including CS)
		i_ep_rd				: IN	STD_LOGIC;	-- RD (including CS)

		-- Host Port signals
		io_had				: INOUT	STD_LOGIC_VECTOR(15 DOWNTO 0);	-- HAD[15..0]
		o_ha16				: OUT	STD_LOGIC;	-- HA16
		i_hp_wr				: IN	STD_LOGIC	-- HWR
	);
END data_path;

ARCHITECTURE data_path OF data_path IS
	SIGNAL temp_buffer		: STD_LOGIC_VECTOR(15 DOWNTO 0);	-- buffer latching HAD

BEGIN
	-- set direction of buffers for accesses between EP, HP and internal PLD resources
	data_flow: PROCESS(i_reset, i_ep_wr, i_ep_rd, i_hp_wr, i_latch_had)
	BEGIN
		-- set default values; only overwritten in CASE statement for valid access
		io_data				<= (OTHERS => 'Z');
		io_had				<= (OTHERS => 'Z');
		o_valid_hp_access	<= DEASSERTED;

		IF (i_reset = DEASSERTED) THEN
			-- address decoding
			CASE i_address IS
				-- address cycle to DM or IO requires left shift by 1 bit on bus HAD
				WHEN HP_DM_ADDRESS | HP_IO_ADDRESS =>
					IF (i_hp_wr = HWR_ASSERTED) THEN
						io_had <= io_data(14 DOWNTO 0) & '0';
						o_ha16 <= io_data(15);
					END IF;
					-- signal address write cycle as valid access
					IF (i_ep_wr = EWR_ASSERTED) THEN
						o_valid_hp_access <= ASSERTED;
					END IF;						
				
				-- address cycle to PM requires left shift by 2 bits on bus HAD
				WHEN HP_PM_ADDRESS =>
					IF (i_hp_wr = HWR_ASSERTED) THEN
						io_had <= io_data(13 DOWNTO 0) & "00";
						o_ha16 <= io_data(14);
					END IF;
					-- signal address write cycle as valid access
					IF (i_ep_wr = EWR_ASSERTED) THEN
						o_valid_hp_access <= ASSERTED;
					END IF;						

				-- data cycles don't require shifts on bus HAD
				WHEN HP_DM_DATA | HP_PM_DATA | HP_IO_DATA =>
					IF (i_hp_wr = HWR_ASSERTED) THEN
						io_had <= io_data;
					ELSIF (i_ep_rd = ERD_ASSERTED) THEN
						IF(i_latch_had = ASSERTED) THEN
							temp_buffer <= io_had;
						ELSE
							io_data <= temp_buffer;
						END IF;
					END IF;
					-- signal data read and write as alid access
					IF (i_ep_wr = EWR_ASSERTED OR i_ep_rd = ERD_ASSERTED) THEN
						o_valid_hp_access <= ASSERTED;
					END IF;

				-- access to register containing version number of PLD
				WHEN REG_VERSION =>
					IF (i_ep_rd = ERD_ASSERTED) THEN
						io_data <= CONV_STD_LOGIC_VECTOR(PLD_VERSION, 16);
					END IF;

				-- use default signals for access to reserved locations
				WHEN OTHERS =>
					NULL;
			END CASE;
		END IF;
	END PROCESS data_flow;
END data_path;
